@@ -2299,6 +2299,24 @@ void CTvheadend::ParseRecordingAddOrUpdate(htsmsg_t* msg, bool bAdd)
       !htsmsg_get_u32(msg, "duplicate", &dup) && dup == 1)
     return;
 
+  /* Ignore recordings without a file (e.g. removed recordings) */
+  const char* error = htsmsg_get_str(msg, "error");
+  if (error && (strstr(error, "missing") != nullptr))
+  {
+    const auto it = m_recordings.find(id);
+    if (it != m_recordings.end())
+    {
+      m_recordings.erase(it);
+
+      if (m_asyncState.GetState() > ASYNC_DVR)
+      {
+        TriggerTimerUpdate();
+        TriggerRecordingUpdate();
+      }
+    }
+    return;
+  }
+
   /* Get/create entry */
   Recording& rec = m_recordings[id];
   Recording comparison = rec;
@@ -2588,15 +2606,12 @@ void CTvheadend::ParseRecordingAddOrUpdate(htsmsg_t* msg, bool bAdd)
   }
 
   /* Error */
-  str = htsmsg_get_str(msg, "error");
-  if (str)
+  if (error)
   {
-    if (!std::strcmp(str, "300"))
+    if (!std::strcmp(error, "300"))
       rec.SetState(PVR_TIMER_STATE_ABORTED);
-    else if (strstr(str, "missing") != nullptr)
-      rec.SetState(PVR_TIMER_STATE_ERROR);
-    else
-      rec.SetError(str);
+
+    rec.SetError(error);
   }
 
   /* A running recording will have an active subscription assigned to it */
@@ -2641,11 +2656,10 @@ void CTvheadend::ParseRecordingAddOrUpdate(htsmsg_t* msg, bool bAdd)
   /* Update */
   if (rec != comparison)
   {
-    std::string error = rec.GetError().empty() ? "none" : rec.GetError();
+    const std::string error = rec.GetError().empty() ? "n/a" : rec.GetError();
 
-    Logger::Log(LogLevel::LEVEL_DEBUG, "recording id:%d, state:%s, title:%s, desc:%s, error:%s",
-                rec.GetId(), state, rec.GetTitle().c_str(), rec.GetDescription().c_str(),
-                error.c_str());
+    Logger::Log(LogLevel::LEVEL_DEBUG, "recording id:%d, state:%s, title:%s, error:%s",
+                rec.GetId(), state, rec.GetTitle().c_str(), error.c_str());
 
     if (m_asyncState.GetState() > ASYNC_DVR)
     {
